from RoboidControl.Messages.Messages import IMessage
from Thing import Thing

from typing import Union, Optional

class ModelUrlMsg(IMessage):
    id = 0x90
    length = 4
    
    def __init__(self, arg1: Union[bytes, int], thing: Optional[Thing] = None):
        if thing is None:
            data = bytes(arg1)
            self.thing_id = data[2]
            # model url length is not needed here
            self.url = data[ModelUrlMsg.length:].decode("utf-8")
        else:
            self.network_id: int = int(arg1)
            self.thing_id: int = thing.id
            self.url: Optional[str] = thing.model_url
    
    def Serialize(self, buffer_ref: list[bytearray]):
        if self.url is None:
            return 0
        
        buffer: bytearray = buffer_ref[0]

        encoded_url = self.url.encode('utf-8')
        url_length = len(encoded_url)
        full_length = ModelUrlMsg.length + url_length
        if url_length == 0 or full_length > len(buffer):
            return 0

        buffer[0:ModelUrlMsg.length] = [
            ModelUrlMsg.id,
            0, # network_id,
            self.thing_id,
            url_length
        ]
        # Append the url string
        buffer[ModelUrlMsg.length:full_length] = encoded_url
        return full_length
    
