from RoboidControl.Messages.Messages import IMessage
from Thing import Thing

from typing import Union, Optional

class NameMsg(IMessage):
    id = 0x91
    length = 4

    def __init__(self, arg1: Union[bytes, int], thing: Optional[Thing] = None):
        if thing is None:
            data = bytes(arg1)
            self.thing_id = data[2]
            # name_length is not needed here
            self.name = data[NameMsg.length:].decode("utf-8")
        else:
            self.network_id: int = int(arg1)
            self.thing_id: int = thing.id
            self.name: Optional[str] = thing.name

    def Serialize(self, buffer_ref: list[bytearray]):
        if self.name is None:
            return 0

        buffer: bytearray = buffer_ref[0]

        encoded_name = self.name.encode('utf-8')
        name_length = len(encoded_name)
        full_length = NameMsg.length + name_length
        if name_length == 0 or full_length > len(buffer):
            return 0
        
        buffer[0:NameMsg.length] = [
            NameMsg.id,
            0, #self.network_id,
            self.thing_id,
            name_length
        ]
        # Append the name string
        buffer[NameMsg.length:full_length] = encoded_name
        return full_length
